@@ -23,7 +23,7 @@
 //! - Because UDP is a "connectionless" protocol, it offers no guarantee of "conversation" between traditional client and server roles. All the UDP transport layer gives you is a method to listen for messages thrown at you, and to throw messages at someone. Owing to this, CoAP machines are expected to perform both client and server roles (or more accurately, _sender_ and _receiver_ roles)
 //! - While _classes_ of status codes are the same (Success 2xx -> 2.xx, Client error 4xx -> 4.xx, Server error 5xx -> 5.xx), the semantics of the individual response codes differ.
 
-#![doc(html_root_url = "https://docs.rs/kwap/0.1.3")]
+#![doc(html_root_url = "https://docs.rs/kwap/0.1.4")]
 #![cfg_attr(all(not(test), feature = "no_std"), no_std)]
 #![cfg_attr(not(test),
             deny(missing_debug_implementations,
@@ -43,14 +43,18 @@ pub mod resp;
 pub mod req;
 
 /// CoAP client
-pub mod event;
-
-/// CoAP client
-pub mod client;
+pub mod core;
 
 /// kwap configuration
 pub mod config;
 
+/// sockets
+pub mod socket;
+
+/// `std`-only kwap stuff
+#[cfg(any(test, not(feature = "no_std")))]
+pub mod std;
+
 static mut ID: u16 = 0;
 
 fn generate_id() -> kwap_msg::Id {
@@ -127,3 +131,39 @@ macro_rules! code {
 pub(crate) use code;
 use kwap_common::Array;
 use kwap_msg::{Opt, OptDelta, OptNumber};
+
+#[cfg(test)]
+pub(crate) mod test {
+  use no_std_net::{SocketAddr, ToSocketAddrs};
+  use socket::*;
+
+  use super::*;
+
+  /// A mocked socket
+  #[derive(Clone, Debug, Default)]
+  pub struct TubeSock(pub Option<SocketAddr>, pub Vec<u8>);
+
+  impl Socket for TubeSock {
+    type Error = Option<()>;
+
+    fn connect<A: ToSocketAddrs>(&mut self, a: A) -> nb::Result<(), Self::Error> {
+      self.0 = a.to_socket_addrs().unwrap().next();
+      Ok(())
+    }
+
+    fn recv(&mut self, buf: &mut [u8]) -> nb::Result<(usize, SocketAddr), Self::Error> {
+      if self.1.is_empty() {
+        return Err(nb::Error::WouldBlock);
+      }
+
+      let n = self.1.len();
+      self.1.drain(..).enumerate().for_each(|(ix, el)| buf[ix] = el);
+      Ok((n, self.0.unwrap()))
+    }
+
+    fn send(&mut self, buf: &[u8]) -> nb::Result<(), Self::Error> {
+      self.1 = buf.iter().copied().collect();
+      Ok(())
+    }
+  }
+}
