@@ -1,10 +1,6 @@
-use core::ops::{Deref, DerefMut};
-
-use kwap_common::Array;
-use kwap_msg::{Message, Opt, OptNumber, Payload, Type};
+use kwap_msg::{EnumerateOptNumbers, Message, Payload, Type};
 #[cfg(feature = "alloc")]
-use std_alloc::{string::{FromUtf8Error, String},
-                vec::Vec};
+use std_alloc::string::{FromUtf8Error, String};
 
 use crate::config::{self, Config};
 
@@ -37,12 +33,12 @@ pub mod code;
 ///
 /// fn start_server(f: impl FnOnce(kwap::req::Req<Alloc>) -> kwap::resp::Resp<Alloc>) {
 ///   // servery things
-/// # f(kwap::req::Req::get("foo"));
+/// # f(kwap::req::Req::get("foo", 0, ""));
 /// }
 /// ```
 #[derive(Clone, Debug)]
 pub struct Resp<Cfg: Config> {
-  msg: config::Message<Cfg>,
+  pub(crate) msg: config::Message<Cfg>,
   opts: Option<Cfg::OptNumbers>,
 }
 
@@ -110,3 +106,19 @@ impl<Cfg: Config> Resp<Cfg> {
     }
   }
 }
+
+impl<Cfg: Config> From<Resp<Cfg>> for config::Message<Cfg> {
+  fn from(mut rep: Resp<Cfg>) -> Self {
+    rep.normalize_opts();
+    rep.msg
+  }
+}
+
+impl<Cfg: Config> From<config::Message<Cfg>> for Resp<Cfg> {
+  fn from(mut msg: config::Message<Cfg>) -> Self {
+    let opts = msg.opts.into_iter().enumerate_option_numbers().collect();
+    msg.opts = Default::default();
+
+    Self { msg, opts: Some(opts) }
+  }
+}
